@@ -2,6 +2,7 @@
 
 #include "iptux/Application.h"
 #include "iptux/TestHelper.h"
+#include "iptux/UiCoreThread.h"
 
 using namespace std;
 using namespace iptux;
@@ -16,5 +17,10 @@ TEST(Application, Constructor) {
   do_action(app, "help.whats_new");
   do_action(app, "tools.open_chat_log");
   do_action(app, "tools.open_system_log");
+
+  PPalInfo pal = make_shared<PalInfo>("127.0.0.1", 2425);
+  app->getCoreThread()->AttachPalToList(pal);
+  app->_ForTestProcessEvents();
+  app->_ForTestProcessEvents();
   DestroyApplication(app);
 }
