@@ -46,6 +46,7 @@ class DialogPeer : public DialogBase {
   void init();
 
   GtkWindow* CreateMainWindow();
+  void CreateTitle();
   GtkWidget* CreateAllArea();
 
   GtkWidget* CreateInfoArea();
@@ -65,8 +66,9 @@ class DialogPeer : public DialogBase {
   void FeedbackMsg(const std::vector<ChipData>& dtlist);
   MsgPara* PackageMsg(const std::vector<ChipData>& dtlist);
   void refreshSendAction();
+  std::string GetTitle();
 
-  //回调处理部分
+  // 回调处理部分
  private:
   static void onRecvTreeSelectionChanged(DialogPeer& self, GtkTreeSelection*);
   static void onAcceptButtonClicked(DialogPeer* self);
@@ -89,6 +91,7 @@ class DialogPeer : public DialogBase {
     DialogBase::AttachFolder(&self);
   }
   static void onRequestSharedResources(void*, void*, DialogPeer& self);
+  static void onPaste(void*, void*, DialogPeer* self);
   static void onSendMessage(void*, void*, DialogPeer& self) {
     DialogBase::SendMessage(&self);
   }
@@ -103,9 +106,9 @@ class DialogPeer : public DialogBase {
  protected:
   GtkApplicationWindow* window;
   std::shared_ptr<IptuxConfig> config;
-  int64_t torcvsize;  //总计待接收大小(包括已接收)
-  int64_t rcvdsize;   //总计已接收大小
-  guint timerrcv;     //接收文件界面更新计时器ID
+  int64_t torcvsize;  // 总计待接收大小(包括已接收)
+  int64_t rcvdsize;   // 总计已接收大小
+  guint timerrcv;     // 接收文件界面更新计时器ID
   GtkWidget* fileToReceiveTreeviewWidget = nullptr;
   gulong sigId = 0;
 };
