@@ -17,6 +17,8 @@ using namespace std;
 
 namespace iptux {
 
+const char* const kObjectKeyImagePath = "image-path";
+
 /**
  * 文件传输树(trans-tree)底层数据结构.
  * 14,0 status,1 task,2 peer,3 ip,4 filename,5 filelength,6 finishlength,7
@@ -622,6 +624,8 @@ static void InsertStringToBuffer(GtkTextBuffer* buffer, const gchar* s) {
   }
   g_match_info_free(matchinfo);
   gtk_text_buffer_insert(buffer, &iter, string + urlendp, -1);
+  gtk_text_buffer_get_end_iter(buffer, &iter);
+  gtk_text_buffer_insert(buffer, &iter, "\n", -1);
 }
 
 /**
@@ -631,7 +635,8 @@ static void InsertStringToBuffer(GtkTextBuffer* buffer, const gchar* s) {
  */
 static void InsertHeaderToBuffer(GtkTextBuffer* buffer,
                                  const MsgPara* para,
-                                 CPPalInfo me) {
+                                 CPPalInfo me,
+                                 time_t now) {
   GtkTextIter iter;
   gchar* header;
 
@@ -640,21 +645,22 @@ static void InsertHeaderToBuffer(GtkTextBuffer* buffer,
    */
   switch (para->stype) {
     case MessageSourceType::PAL:
-      header = getformattime(FALSE, "%s", para->getPal()->getName().c_str());
+      header =
+          getformattime2(now, FALSE, "%s", para->getPal()->getName().c_str());
       gtk_text_buffer_get_end_iter(buffer, &iter);
       gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, header, -1,
                                                "pal-color", NULL);
       g_free(header);
       break;
     case MessageSourceType::SELF:
-      header = getformattime(FALSE, "%s", me->getName().c_str());
+      header = getformattime2(now, FALSE, "%s", me->getName().c_str());
       gtk_text_buffer_get_end_iter(buffer, &iter);
       gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, header, -1,
                                                "me-color", NULL);
       g_free(header);
       break;
     case MessageSourceType::ERROR:
-      header = getformattime(FALSE, "%s", _("<ERROR>"));
+      header = getformattime2(now, FALSE, "%s", _("<ERROR>"));
       gtk_text_buffer_get_end_iter(buffer, &iter);
       gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, header, -1,
                                                "error-color", NULL);
@@ -663,36 +669,33 @@ static void InsertHeaderToBuffer(GtkTextBuffer* buffer,
     default:
       break;
   }
+  gtk_text_buffer_get_end_iter(buffer, &iter);
+  gtk_text_buffer_insert(buffer, &iter, "\n", -1);
 }
 
-#define OCCUPY_OBJECT 0x01
-
 /**
- * 插入图片到TextBuffer(非UI线程安全).
+ * 插入图片到TextBuffer.
  * @param buffer text-buffer
  * @param path 图片路径
  */
 static void InsertPixbufToBuffer(GtkTextBuffer* buffer, const gchar* path) {
-  GtkTextIter start, end;
-  GdkPixbuf* pixbuf;
+  GtkTextIter iter;
 
-  if ((pixbuf = gdk_pixbuf_new_from_file(path, NULL))) {
-    gtk_text_buffer_get_start_iter(buffer, &start);
-    if (gtk_text_iter_get_char(&start) == OCCUPY_OBJECT ||
-        gtk_text_iter_forward_find_char(
-            &start, GtkTextCharPredicate(giter_compare_foreach),
-            GUINT_TO_POINTER(OCCUPY_OBJECT), NULL)) {
-      end = start;
-      gtk_text_iter_forward_char(&end);
-      gtk_text_buffer_delete(buffer, &start, &end);
-    }
-    gtk_text_buffer_insert_pixbuf(buffer, &start, pixbuf);
-    g_object_unref(pixbuf);
-  }
+  gtk_text_buffer_get_end_iter(buffer, &iter);
+  GtkTextChildAnchor* anchor = gtk_text_child_anchor_new();
+  g_object_set_data_full(G_OBJECT(anchor), kObjectKeyImagePath, g_strdup(path),
+                         GDestroyNotify(g_free));
+  gtk_text_buffer_insert_child_anchor(buffer, &iter, anchor);
+  gtk_text_buffer_get_end_iter(buffer, &iter);
+  gtk_text_buffer_insert(buffer, &iter, "\n", -1);
 }
 
 void GroupInfo::addMsgPara(const MsgPara& para) {
-  GtkTextIter iter;
+  time_t now = time(NULL);
+  _addMsgPara(para, now);
+}
+
+void GroupInfo::_addMsgPara(const MsgPara& para, time_t now) {
   const gchar* data;
 
   time(&last_activity_);
@@ -702,18 +705,15 @@ void GroupInfo::addMsgPara(const MsgPara& para) {
     data = chipData->data.c_str();
     switch (chipData->type) {
       case MESSAGE_CONTENT_TYPE_STRING:
-        InsertHeaderToBuffer(buffer, &para, me);
-        gtk_text_buffer_get_end_iter(buffer, &iter);
-        gtk_text_buffer_insert(buffer, &iter, "\n", -1);
+        InsertHeaderToBuffer(buffer, &para, me, now);
         InsertStringToBuffer(buffer, data);
-        gtk_text_buffer_get_end_iter(buffer, &iter);
-        gtk_text_buffer_insert(buffer, &iter, "\n", -1);
         last_message_ = StrFirstNonEmptyLine(chipData->data);
         if (logSystem) {
           logSystem->communicateLog(&para, "[STRING]%s", data);
         }
         break;
       case MESSAGE_CONTENT_TYPE_PICTURE:
+        InsertHeaderToBuffer(buffer, &para, me, now);
         InsertPixbufToBuffer(buffer, data);
         last_message_ = _("[IMG]");
         if (logSystem) {
